/*
 *  $Id: 2ndpass.c,v 1.5 2001/04/11 03:19:47 davej Exp $
 * This file is part of Sourcery.
 * (C) 2001 Dave Jones.
 *
 * Licensed under the terms of the GNU GPL License version 2.
 */

#include <stdio.h>
#include <gtk/gtk.h>
#include "sourcery.h"
#include "objdump.h"
#include "bfd.h"

static char *blanknode[4];
static char emptystring[]="";


extern GtkWidget *clist1;

static void parsenode_IA32_pre (char *columns[4], struct objdump_disasm_info __unused *aux, bfd_byte *data)
{
	char ret_str[]="** End of function **";
	char breakpt_str[]="** breakpoint **";
	char cld_str[]="Clear direction flag";
	unsigned char byte1;

	if (data==NULL)
		return;

	byte1 = data[0];

	if ((byte1==0xC2) || (byte1==0xC3))
		sprintf (columns[3], "%s", ret_str);

	if (byte1==0xCC)
		sprintf (columns[3], "%s", breakpt_str);

	if (byte1==0xFC)
		sprintf (columns[3], "%s", cld_str);
}

static void parsenode_IA32_post (char __unused *columns[4], struct objdump_disasm_info __unused *aux, bfd_byte *data)
{
	unsigned char byte1;

	if (data==NULL)
		return;

	byte1 = data[0];

	if ((byte1==0xC2) || (byte1==0xC3)) {
		blanknode[0]=emptystring;
		blanknode[1]=emptystring;
		blanknode[2]=emptystring;
		blanknode[3]=emptystring;
		gtk_clist_append (GTK_CLIST(clist1), blanknode);
	}
}



/*
 * This routine adds any info it can to the comments field,
 * and also performs extra formatting (blank lines etc)
 * where appropriate.
 */

void parsenode (char *columns[4], struct objdump_disasm_info *aux, bfd_byte *data)
{
	if (columns == NULL) {
		blanknode[0]=emptystring;
		blanknode[1]=emptystring;
		blanknode[2]=emptystring;
		blanknode[3]=emptystring;
		gtk_clist_append (GTK_CLIST(clist1), blanknode);
		return;
	}

	/* Do IA32 specific features. */
	if (bfd_get_arch(aux->abfd) == bfd_arch_i386)
		parsenode_IA32_pre(columns, aux, data);

	gtk_clist_append (GTK_CLIST(clist1), columns);

	/* Clean the comments field ready for next time. */
	memset (columns[3], 0, 128);

	/* Post process. The string is now in the list.
	 * Now we can add any additional strings after it. */

	/* Do IA32 specific features. */
	if (bfd_get_arch(aux->abfd) == bfd_arch_i386)
		parsenode_IA32_post(columns, aux, data);

	/* Clean the comments field ready for next time. */
	memset (columns[3], 0, 128);
}

